
#include <stdlib.h>

#define FALSE 0
#define TRUE 1

int AvailCPUID (void)
{
    /* Tests for availability of CPUID by flipping ID bit eflags */
    int f=FALSE;
    __asm
        {
        pushfd
        pop eax
        mov ebx,eax
        xor eax,0x00200000
        push eax
        popfd
        pushfd
        pop eax
        push ebx
        popfd
        xor eax,ebx
        mov f,eax
        }

    return f;
}

int FirstCPUID (char *s)
{
    int Level;
    // Find processor manufacturer and number of supported CPUID modes
    __asm
        {
        xor eax,eax
        cpuid
        mov Level,eax
        mov esi,s
        mov dword ptr [esi],ebx
        mov dword ptr 4[esi],edx
        mov dword ptr 8[esi],ecx
        }
    s [12] = 0;
    return Level;
}

int FirstAMDCPUID (void)
{
    int Level;
    // Finds number of supported AMD extended CPUID modes
    __asm
        {
        mov eax,80000000H
        cpuid
        mov Level,eax
        }
    return Level;
}

unsigned int VersionCPUID (void)
{
    unsigned int Features;
    // Finds features of CPU
    __asm
        {
        mov eax,1
        cpuid
        mov Features,edx
        }
    return Features;
}

int FamilyCPUID (void)
{
    unsigned int Family;
    // Finds family of CPU
    __asm
        {
        mov eax,1
        cpuid
        mov Family,eax
        }
    return Family;
}

unsigned int AMDVersionCPUID (void)
{
    unsigned int Features;
    // Finds AMD extended features of CPU
    __asm
        {
        mov eax,80000001H
        cpuid
        mov Features,edx
        }
    return Features;
}

/*int isSSEOS (void)
{
    int Flag;
    // Checks operating system supports SSE
    __asm
        {
        mov eax,1
        cpuid            // This is needed or the next instruction doesn't work
                        // No idea why
        mov eax,cr4
        shr eax,9
        and eax,1
        mov Flag,eax
        }
    return Flag;
}*/

int isPentium (void)
{
   unsigned int Features;
    // Returns TRUE if processor is a Pentium or compatible.
    if (!AvailCPUID ()) return FALSE;       // no CPUID
    if (FamilyCPUID () < 5) return FALSE;   // must be 486
    Features = VersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    return TRUE;
}

int isPentiumMMX (void)
{
   unsigned int Features;
    // Returns TRUE if processor is a Pentium with MMX or compatible.
    if (!AvailCPUID ()) return FALSE;
    if (FamilyCPUID () < 5) return FALSE;
    Features = VersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    if (!(Features & 0x800000)) return FALSE; // no MMX
    return TRUE;
}

int isPentiumPro (void)
{
    unsigned int Features;
    // Returns TRUE if processor is a Pentium Pro or compatible.
    if (!AvailCPUID ()) return FALSE;
    if (FamilyCPUID () < 6) return FALSE;
    Features = VersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    if (!(Features & 0x8000)) return FALSE; // no CMOV/FCMOV/FCOMI
    return TRUE;
}

int isPentiumII (void)
{
    unsigned int Features;
    // Returns TRUE if processor is a Pentium II or compatible.
    if (!AvailCPUID ()) return FALSE;
    if (FamilyCPUID () < 6) return FALSE;
    Features = VersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    if (!(Features & 0x8000)) return FALSE; // no CMOV/FCMOV/FCOMI
    if (!(Features & 0x800000)) return FALSE; // no MMX
    return TRUE;
}

int isPentiumIII (void)
{
   unsigned int Features;
    // Returns TRUE if processor is a Pentium III or compatible.
    // Returns FALSE if Pentium III with no operating system support
    if (!AvailCPUID ()) return FALSE;
    if (FamilyCPUID () < 6) return FALSE;
    Features = VersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    if (!(Features & 0x8000)) return FALSE; // no CMOV/FCMOV/FCOMI
    if (!(Features & 0x800000)) return FALSE; // no MMX
    if (!(Features & 0x2000000)) return FALSE; // no Streaming SIMD
    //if (!(isSSEOS ())) return FALSE; // no Streaming SIMD operating system support
    return TRUE;
}

int isAMDK6 (void)
{
    char Brand [64];
    unsigned int Features;
    // Returns TRUE if processor is an AMD K6 with MMX or compatible.
    // Returns FALSE if processor is a Pentium MMX, even though this
    // is a compatible processor. Use isPentiumMMX for that.
    if (!AvailCPUID ()) return FALSE;
    if (FamilyCPUID () < 5) return FALSE;    // not Pentium class or above
    if (FirstCPUID (Brand) < 1) return FALSE;
    if (FirstAMDCPUID () < 0x80000001) return FALSE;
    Features = AMDVersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    if (!(Features & 0x800000)) return FALSE; // no MMX
    return TRUE;
}

int isAMDK62 (void)
{
    char Brand [64];
    unsigned int Features;
    // Returns TRUE if processor is an AMD K6 with MMX and 3DNow! or compatible.
    if (!AvailCPUID ()) return FALSE;
    if (FamilyCPUID () < 5) return FALSE;    // not Pentium class or above
    if (FirstCPUID (Brand) < 1) return FALSE;
    if (FirstAMDCPUID () < 0x80000001) return FALSE;
    Features = AMDVersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    if (!(Features & 0x800000)) return FALSE; // no MMX
    if (!(Features & 0x80000000)) return FALSE; // no 3DNow!
    return TRUE;
}

int isAthlon (void)
{
    char Brand [64];
    unsigned int Features;
    // Returns TRUE if processor is an AMD Athlon with MMX and Advanced 3DNow! or compatible.
    if (!AvailCPUID ()) return FALSE;
    if (FamilyCPUID () < 6) return FALSE;    // not Pentium Pro class or above
    if (FirstCPUID (Brand) < 1) return FALSE;
    if (FirstAMDCPUID () < 0x80000001) return FALSE;
    Features = AMDVersionCPUID ();
    if (!(Features & 0x1)) return FALSE;    // no float
    if (!(Features & 0x8000)) return FALSE; // no CMOV/FCMOV/FCOMI
    if (!(Features & 0x800000)) return FALSE; // no MMX
    if (!(Features & 0x400000)) return FALSE; // no K7 MMX
    if (!(Features & 0x80000000)) return FALSE; // no 3DNow!
    if (!(Features & 0x40000000)) return FALSE; // no Advanced 3DNow!
    return TRUE;
}


#define CHECKFEATURE_ERROR  0x10000001	/* cpu feature check had a problem, call cpuFeatures */
#define CHECKFEATURE_X87FPU 0x00000001  /* cpu has x87 fpu */
#define CHECKFEATURE_VMODEE 0x00000002  /* cpu has Virtual 8086 mode enhancement */
#define CHECKFEATURE_DEBEXT 0x00000004  /* cpu has debugging extensions */
#define CHECKFEATURE_PGSEXT	0x00000008  /* cpu has page extensions */
#define CHECKFEATURE_TSCOUN	0x00000010  /* cpu has time stamp counter */
#define CHECKFEATURE_XXMSRS	0x00000020  /* cpu has RDMSR/WRMSR inst support */
#define CHECKFEATURE_PHAEXT	0x00000040  /* cpu has physical address extension */
#define CHECKFEATURE_MCHEXT	0x00000080  /* cpu has machine check exception */
#define CHECKFEATURE_CMPXCH	0x00000100  /* cpu has 8 byte cmp and xchg inst support */
#define CHECKFEATURE_APICON	0x00000200  /* cpu has APIC on chip */
#define CHECKFEATURE_SYSXXX	0x00000800  /* cpu has SYSENTER/SYSLEAVE */
#define CHECKFEATURE_MTRREG	0x00001000  /* cpu has memory type range registers */
#define CHECKFEATURE_PTEBIT	0x00002000  /* cpu has global PTE bit */
#define CHECKFEATURE_MCKARC	0x00004000  /* cpu has machine check architexture */
#define CHECKFEATURE_CMOVIN	0x00008000  /* cpu has CMOV inst support */
#define CHECKFEATURE_PGATTT	0x00010000  /* cpu has page attribute table */
#define CHECKFEATURE_PGSEX2	0x00020000  /* cpu has page size extension */
#define CHECKFEATURE_SERNUM	0x00040000  /* cpu has processor serial number */
#define CHECKFEATURE_CFLUSH	0x00080000  /* processor supports CFLUSH inst */
#define CHECKFEATURE_DBSTOR	0x00200000  /* cpu has debug store */
#define CHECKFEATURE_ACPICT	0x00400000  /* ACPI on processor */
#define CHECKFEATURE_MMXCMP	0x00800000  /* cpu has MMX technology */
#define CHECKFEATURE_FXXSTO	0x01000000	/* cpu supports FXSAVE/FXRSTORE inst */
#define CHECKFEATURE_SSEEXT	0x02000000  /* cpu has SSE extensions */
#define CHECKFEATURE_SSE2EX 0x04000000  /* cpu has sse2 extensions */
#define CHECKFEATURE_SSNOOP	0x08000000  /* cpu supports system snoop */
#define CHECKFEATURE_PTHERM	0x10000000  /* cpu has process thermomitor */

#define CHECKFEATURE_SSE3EX 0x00000001  /* SSE 3 Extension EXTENDED FLAG !!! */

unsigned int cpuFeatures (unsigned int eaxval = 1)
{
	unsigned int features;
	__asm
	{
		mov eax, eaxval			;set request type to a feature list from processor
		cpuid				;send request
		lea ebx, features	
		mov [ebx], edx		;save value
	}

	//cpufeatures = features; 
	return features;
}

unsigned int cpuFeaturesExt (unsigned int eaxval = 1)
{
	unsigned int features;
	__asm
	{
		mov eax, eaxval			;set request type to a feature list from processor
		cpuid				;send request
		lea ebx, features	
		mov [ebx], ecx		;save value
	}

	//cpufeatures = features; 
	return features;
}

unsigned int cpuCheckFeature (unsigned int feature, bool extended = false)
{
	if((extended ? cpuFeaturesExt() : cpuFeatures()) & feature) /* feature exists, good- tell user */
		return feature;

	return 0;
}

bool HasSSE()
{
	if (isPentiumIII == 0)
	{
		return(false);
	}
	if (cpuCheckFeature(CHECKFEATURE_SSEEXT) == CHECKFEATURE_SSEEXT)
	{
		return(true);
	}
	else
	{
		return(false);
	}
}

bool HasSSE2()
{
	if (!HasSSE())
	{
		return(false);
	}
	if (cpuCheckFeature(CHECKFEATURE_SSE2EX) == CHECKFEATURE_SSE2EX)
	{
		return(true);
	}
	else
	{
		return(false);
	}
}

bool HasSSE3()
{
	if (!HasSSE2())
	{
		return(false);
	}
    if (FirstAMDCPUID () < 0x80000001) return FALSE;
	if (cpuCheckFeature(CHECKFEATURE_SSE3EX, true) == CHECKFEATURE_SSE3EX)
	{
		return(true);
	}
	else
	{
		return(false);
	}
}